@@ -15,22 +15,30 @@ Myghty template names can be used with ``/`` and file extensions.
 The render functions are intended as the primary user-visible rendering 
 commands and hook into Buffet to make rendering content easy.
 """
-import pkg_resources
+import logging
 import os
+import warnings
+
 try:
     from cStringIO import StringIO
 except ImportError:
     from StringIO import StringIO
-import paste.wsgiwrappers
+import pkg_resources
+
 import pylons
 
+__all__ = ['Buffet', 'MyghtyTemplatePlugin', 'render', 'render_response']
+
 PYLONS_VARS = ['c', 'g', 'h', 'render', 'request', 'session', 'translator',
                'ungettext', '_', 'N_']
 
+log = logging.getLogger(__name__)
+
 class BuffetError(Exception):
     """Buffet Exception"""
     pass
 
+
 class Buffet(object):
     """Buffet style plug-in template rendering
     
@@ -38,7 +46,7 @@ class Buffet(object):
     `Buffet Project <http://projects.dowski.com/projects/buffet>`_ from which
     this class inherits its name.
     """
-    def __init__(self, default_engine=None, template_root=None, 
+    def __init__(self, default_engine=None, template_root=None,
         default_options=None, **config):
         """Initialize the Buffet renderer, and optionally set a default
         engine/options"""
@@ -48,6 +56,7 @@ class Buffet(object):
         self.template_root = template_root
         self.default_options = default_options
         self.engines = {}
+        log.debug("Initialized Buffet object")
         if self.default_engine:
             self.prepare(default_engine, template_root, **config)
         
@@ -57,20 +66,21 @@ class Buffet(object):
         This method must be run before the `render <#render>`_ method is called
         so that the ``template_root`` and options can be set. Template engines
         can also be aliased if you wish to use multiplate configurations of the
-        same template engines, or prefer a shorter name when rendering a template
-        with the engine of your choice.
+        same template engines, or prefer a shorter name when rendering a
+        template with the engine of your choice.
         """
         Engine = available_engines.get(engine_name, None)
         if not Engine:
             raise TemplateEngineMissing('Please install a plugin for '
                 '"%s" to use its functionality' % engine_name)
         engine_name = alias or engine_name
-        defaults = config.pop('default_options', None)
-        extra_vars_func = config.get('extra_vars_func', None)
+        extra_vars_func = config.pop(engine_name + '.extra_vars_func', None)
         self.engines[engine_name] = \
             dict(engine=Engine(extra_vars_func=extra_vars_func,
                                options=config), 
                  root=template_root)
+        log.debug("Adding %s template language for use with Buffet", 
+                  engine_name)
     
     def _update_names(self, ns):
         """Return a dict of Pylons vars and their respective objects updated
@@ -78,16 +88,21 @@ class Buffet(object):
         d = dict(
             c=pylons.c._current_obj(),
             g=pylons.g._current_obj(),
-            h=pylons.h._current_obj(),
+            h=pylons.config.get('pylons.h') or pylons.h._current_obj(),
             render=render,
             request=pylons.request._current_obj(),
-            session=pylons.session._current_obj(),
             translator=pylons.translator,
             ungettext=pylons.i18n.ungettext,
             _=pylons.i18n._,
             N_=pylons.i18n.N_
             )
+        
+        # If the session was overriden to be None, don't populate the session
+        # var
+        if pylons.config['pylons.environ_config'].get('session', True):
+            d['session'] = pylons.session._current_obj()
         d.update(ns)
+        log.debug("Updated render namespace with pylons vars: %s", d)
         return d
     
     def render(self, engine_name=None, template_name=None,
@@ -137,8 +152,8 @@ class Buffet(object):
         engine_config = self.engines.get(engine_name)
         
         if not engine_config:
-            raise Exception, \
-                "No engine with that name configured: %s" % engine_name
+            raise Exception("No engine with that name configured: %s" % \
+                                engine_name)
         
         full_path = template_name
                 
@@ -151,7 +166,10 @@ class Buffet(object):
                 if key in namespace:
                     options[key] = namespace.pop(key)
 
-            namespace['_global_args'] = self._update_names({})
+            if include_pylons_variables:
+                namespace['_global_args'] = self._update_names({})
+            else:
+                namespace['_global_args'] = {}
             
             # If they passed in a variable thats listed in the global_args,
             # update the global args one instead of duplicating it
@@ -167,14 +185,20 @@ class Buffet(object):
                                       '``include_pylons_variables`` is False')
                 else:
                     namespace = self._update_names({})
-            else:
+            elif include_pylons_variables:
                 namespace = self._update_names(namespace)
             
-            if not engine_name.startswith('pylons') and not \
-                    engine_name.startswith('mako'):
+            if not full_path.startswith(os.path.sep) and not \
+                    engine_name.startswith('pylons') and not \
+                    engine_name.startswith('mako') and \
+                    engine_config['root'] is not None:
                 full_path = os.path.join(engine_config['root'], template_name)
                 full_path = full_path.replace(os.path.sep, '.').lstrip('.')
         
+        # Don't pass format into the template engine if it's None
+        if 'format' in options and options['format'] is None:
+            del options['format']
+        
         # If one of them is not None then the user did set something
         if cache_key is not None or cache_expire is not None or cache_type \
             is not None:
@@ -185,23 +209,30 @@ class Buffet(object):
             if cache_expire == 'never':
                 cache_expire = None
             def content():
+                log.debug("Cached render running for %s", full_path)
                 return engine_config['engine'].render(namespace, 
                     template=full_path, **options)
             tfile = full_path
             if options.get('fragment', False):
                 tfile += '_frag'
+            if options.get('format', False):
+                tfile += options['format']
+            log.debug("Using render cache for %s", full_path)
             mycache = pylons.cache.get_cache(tfile)
             content = mycache.get_value(cache_key, createfunc=content, 
                 type=cache_type, expiretime=cache_expire)
             return content
         
+        log.debug("Rendering template %s with engine %s", full_path, engine_name)
         return engine_config['engine'].render(namespace, template=full_path, 
             **options)
-        
+
+
 class TemplateEngineMissing(Exception):
     """Exception to toss when an engine is missing"""
     pass
 
+
 class MyghtyTemplatePlugin(object):
     """Myghty Template Plugin
     
@@ -250,22 +281,43 @@ class MyghtyTemplatePlugin(object):
                                  **optional_args)
         return buf.getvalue()
 
+
 available_engines = {}
 
-for entry_point in pkg_resources.iter_entry_points('python.templating.engines'):
-    Engine = entry_point.load()
-    available_engines[entry_point.name] = Engine
+
+for entry_point in \
+        pkg_resources.iter_entry_points('python.templating.engines'):
+    try:
+        Engine = entry_point.load()
+        available_engines[entry_point.name] = Engine
+    except:
+        import sys
+        from pkg_resources import DistributionNotFound
+        # Warn when there's a problem loading a Buffet plugin unless it's
+        # pylonsmyghty reporting there's no Myghty installed
+        if not isinstance(sys.exc_info()[1], DistributionNotFound) or \
+                entry_point.name != 'pylonsmyghty':
+            import traceback
+            import warnings
+            tb = StringIO()
+            traceback.print_exc(file=tb)
+            warnings.warn("Unable to load template engine entry point: '%s': "
+                          "%s" % (entry_point, tb.getvalue()), RuntimeWarning,
+                          2)
+
 
 def render(*args, **kargs):
     """Render a template and return it as a string (possibly Unicode)
     
     Optionally takes 3 keyword arguments to use caching supplied by Buffet.
     
-    Example::
+    Examples:
         
-        >>> content = render('/my/template.myt')
-        >>> print content
-        >>> content = render('/my/template.myt', fragment=True)
+    .. code-block:: Python
+
+        content = render('/my/template.mako')
+        print content
+        content = render('/my/template2.myt', fragment=True)
     
     .. admonition:: Note
         
@@ -275,17 +327,21 @@ def render(*args, **kargs):
         inheriting any site skin.
     """
     fragment = kargs.pop('fragment', False)
+    format = kargs.pop('format', None)
     args = list(args)
     template = args.pop()
     cache_args = dict(cache_expire=kargs.pop('cache_expire', None), 
                        cache_type=kargs.pop('cache_type', None),
                        cache_key=kargs.pop('cache_key', None))
+    log.debug("Render called with %s args and %s keyword args", args, kargs)
     if args: 
         engine = args.pop()
         return pylons.buffet.render(engine, template, fragment=fragment,
-                                    namespace=kargs, **cache_args)
+                                    format=format, namespace=kargs, 
+                                    **cache_args)
     return pylons.buffet.render(template_name=template, fragment=fragment,
-                                namespace=kargs, **cache_args)
+                                format=format, namespace=kargs, **cache_args)
+
 
 def render_response(*args, **kargs):
     """Returns the rendered response within a Response object
@@ -297,18 +353,21 @@ def render_response(*args, **kargs):
     .. code-block:: Python
         
         def view(self):
-            return render_response('/my/template.myt')
+            return render_response('/my/template.mako')
     """
-    response = pylons.Response(render(*args, **kargs))
+    warnings.warn(pylons.legacy.render_response_warning,
+                  PendingDeprecationWarning, 2)
+
+    response = pylons.response._current_obj()
+    response.content = render(*args, **kargs)
     output_encoding = kargs.get('output_encoding')
     encoding_errors = kargs.get('encoding_errors')
     if output_encoding:
         response.headers['Content-Type'] = '%s; charset=%s' % \
-            (paste.wsgiwrappers.settings['content_type'], output_encoding)
+            (pylons.Response.defaults.get('content_type',
+                                          'text/html'), output_encoding)
     if encoding_errors:
         response.encoding_errors = encoding_errors
     return response
-
-
-__pudge_all__ = ['render', 'render_response', 'Buffet', 'MyghtyTemplatePlugin']
-__all__ = ['render', 'render_response', 'Buffet', 'MyghtyTemplatePlugin']
+render_response.__doc__ = 'Pending Deprecation: %s.\n\n%s' % \
+    (pylons.legacy.render_response_warning, render_response.__doc__)
